// ==================================================================
// Test.java  Moladlib test and demonstration
// (C) 2003, Michael Glickman
// ------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ==================================================================

package com.palmcrust.moladlibtest;

import javax.microedition.midlet.MIDlet;
import java.util.Vector;
import com.palmcrust.moladlib.*;

public class Test extends MIDlet
{

	public Test()
	{
		doTest();	
	}

	public void startApp() 
	{
		notifyDestroyed();
	}

	public void pauseApp() 
	{	}

	public void destroyApp(boolean unc) 
	{	}


	//===============================================================================
	// In order to run a test for a particular secular date, specify 
	// Target-Date: ddmmyyyy  in Moladliblest.jad
	
	private void doTest()
	{
		EventData.resetDisplaySettings();

		DateHolder dh = new DateHolder();
		DateData dd;

		String textDate = getAppProperty("Target-Date");
		
		if (textDate != null && textDate.length() > 4)
		{
			int day = Integer.parseInt(textDate.substring(0,2));
			int month = Integer.parseInt(textDate.substring(2,4))-1;
			int year = Integer.parseInt(textDate.substring(4));
			D("Day " + day + " monbth = " + month + " year = " + year);
			dh.acceptSecularDate(new DateData(day, month, year));
			D("Jad Date");
		} else {
			dh.acceptCurrentDate();
			D("Today is");
		}
		
		
		dd = dh.dateHebrew;
		D("\tHebrew Date: " + dateToTextHebrew(dh));
		D("\tCommon Date: " + dateToTextCommon(dh));
		D("\tAbsolute Date: " + dh.dateAbsolute);

		D("");

		Vector events;

		showCurrentEvents("Today's events", EventData.getEventNamesForDate(dh));
		listEvents("Coming events", EventData.getListOfEvents(dh, false), 5);
		listEvents("Past events", EventData.getListOfEvents(dh, true), 5);
	}
	

	private void showCurrentEvents(final String header, final Vector eventNames)
	{
		int count = eventNames.size();
		if (count <= 0) return;

		
		D(header);		
		for (int i=0; i<count; i++)
			D("\t" + ((String) eventNames.elementAt(i)).substring(1));	
	
		D("");	
	}

	private void listEvents(final String header, final Vector events, int maxCount)
	{
		int count = Math.min( events.size(), maxCount);
		if (count <= 0) return;
		
		D(header);		
		EventData ed;
		String name;
		
		for (int i=0; i<count; i++) {
			ed = (EventData) events.elementAt(i);			
			// We use long event name
			name = EventData.getHolidayName(ed.eventNameIndex, 0) + ":  " + dateToTextCommon(ed.eventDate);	
			if (ed.isKatan) name += " Katan";
			D("\t" + name);
		}	
	
		D("");	
	}

	String dateToTextCommon(final DateHolder dh)
	{
		final DateData dd = dh.dateSecular;
		return  dd.day + " " + SecularCalendar.getMonthName(dd.month) + " " + dd.year;
	}

	String dateToTextHebrew(final DateHolder dh)
	{
		final DateData dd = dh.dateHebrew;
		return  dd.day + " " + HebCalendar.getMonthName(dd.month) + " " + dd.year;
	}


	private void D(final String text)
	{
		System.out.println(text);
		System.out.flush();
	}







}